/*
 * Microsoft.ApplicationInsights, 3.3.8
 * Copyright (c) Microsoft and contributors. All rights reserved.
 *
 * Microsoft Application Insights Team
 * https://github.com/microsoft/ApplicationInsights-JS#readme
 *
 * ---------------------------------------------------------------------------
 * This is a single combined (rollup) declaration file for the package,
 * if you require a namespace wrapped version it is also available.
 * - Namespaced version: types/applicationinsights-clickanalytics-js.namespaced.d.ts
 * ---------------------------------------------------------------------------
 */

import { BaseTelemetryPlugin } from '@microsoft/applicationinsights-core-js';
import { IAppInsightsCore } from '@microsoft/applicationinsights-core-js';
import { IConfig } from '@microsoft/applicationinsights-common';
import { IConfiguration } from '@microsoft/applicationinsights-core-js';
import { ICustomProperties } from '@microsoft/applicationinsights-core-js';
import { IEventTelemetry } from '@microsoft/applicationinsights-common';
import { IPlugin } from '@microsoft/applicationinsights-core-js';
import { IProcessTelemetryContext } from '@microsoft/applicationinsights-core-js';
import { ITelemetryItem } from '@microsoft/applicationinsights-core-js';
import { ITelemetryPluginChain } from '@microsoft/applicationinsights-core-js';

export declare function BehaviorEnumValidator(enumObj: any): (key: string) => any;

export declare function BehaviorMapValidator(map: any): (key: string) => any;

export declare function BehaviorValueValidator(behaviorArray: string[]): (key: string) => string;

export declare class ClickAnalyticsPlugin extends BaseTelemetryPlugin {
    identifier: string;
    priority: number;
    static Version: string;
    constructor();
    initialize(config: IConfiguration & IConfig, core: IAppInsightsCore, extensions: IPlugin[], pluginChain?: ITelemetryPluginChain): void;
    processTelemetry(env: ITelemetryItem, itemCtx?: IProcessTelemetryContext): void;
    /**
     * Logs a page action event.
     * @param IPageActionTelemetry - The page action event to log.
     * @param customProperties - Additional data used to filter events and metrics. Defaults to empty.
     */
    trackPageAction(pageAction?: IPageActionTelemetry, customProperties?: ICustomProperties): void;
}

/**
 * ClickAnalytics Configuration
 */
export declare interface IClickAnalyticsConfiguration {
    /**
     * Automatic capture configuration. Default is true
     */
    autoCapture?: boolean;
    /**
     * Callbacks configuration
     */
    callback?: IValueCallback;
    /**
     * Page tags
     */
    pageTags?: {
        [name: string]: string | number | boolean | string[] | number[] | boolean[] | object;
    };
    /**
     * Core data configuration
     */
    coreData?: ICoreData;
    /**
     * Custom Data Tags provided to ovverride default tags used to capture click data.
     */
    dataTags?: ICustomDataTags;
    /**
     * Validator to use for the data-bhvr value
     */
    behaviorValidator?: (value: string | number) => string | number;
    /**
     * Default Behavior value when Right Click event has occured. This
     * value will be ovverriden if the element has the data-*-bhvr attribute present.
     */
    defaultRightClickBhvr?: string | number;
    /**
     * Flag to drop events that do not have custom event names, no parentId and no data in content (basically no useful click data).
     * Default will be false
     */
    dropInvalidEvents?: boolean;
    /**
     * Enables the logging of values after a "#" character of the URL. Default is "false."
     */
    urlCollectHash?: boolean;
    /**
     * Enables the logging of the query string of the URL. Default is "false."
     */
    urlCollectQuery?: boolean;
    /**
     * A list of element types to track. Default is "undefined" which means default elements ["a", "button", "area", "input"] are tracked.
     * If set, it will combine with the default element types.
     */
    trackElementTypes?: string;
}

/**
 * Core data configuration
 */
declare interface ICoreData {
    /**
     * document.referrer is the default. This is used to override the default value.
     */
    referrerUri?: string;
    /**
     * window.location.href is the default. This is used to override the default value.
     */
    requestUri?: string;
    /**
     * Default page name is derived from the url. This is used to override the default.
     */
    pageName?: string;
    /**
     * PageType is captured from a meta tag named awa-pageType. This is used to override the value or bypass defining meta tags.
     */
    pageType?: string;
}

/**
 * Custom Data Tags Configuration
 */
declare interface ICustomDataTags {
    /**
     * When a particular element is not tagged with content name prefix or content name prefix is not provided by user, this flag is used to collect standard HTML attribute for contentName and id.
     */
    useDefaultContentNameOrId?: boolean;
    /**
     * Automatic capture content name and value of elements which are tagged with provided prefix
     */
    customDataPrefix?: string;
    /**
     * Click Analytics supports a JSON blob content meta data tagging instead of individual data-* attributes. The default attribute is data-ai-blob. This property allows for changing that attribute name.
     */
    aiBlobAttributeTag?: string;
    /**
     * Automatic capture metadata name and content with provided prefix
     */
    metaDataPrefix?: string;
    /**
     * Automatic capture all metadata names and content. Default is false. If enabled this will override provided metaTagPrefix.
     */
    captureAllMetaDataContent?: boolean;
    /**
     * Stop traversing up the DOM to capture content name and value of elements when encountered with this tag
     */
    parentDataTag?: string;
    /**
     * Custom attribute Tag to not track telemetry data
     */
    dntDataTag?: string;
}

/**
 * Page Action event
 */
declare interface IPageActionTelemetry extends IEventTelemetry {
    /**
     * Target uri for PageAction events
     */
    targetUri?: string;
    /**
     *  One of the awa.actionType values
     */
    actionType?: string;
    /**
     * One of the awa.behavior values.
     */
    behavior?: string | number;
    /**
     * X, Y representing the absolute co-ordinates withrespect to topleft corner of the page. This should be normalized for the screen resolution to provide better heat map.
     */
    clickCoordinates?: string;
    /**
     * JSON-formatted array of content acted upon
     */
    content?: any;
    /**
     * Version indicating content version which aids in parsing the content.
     */
    contentVer?: string;
    /**
     * Uri of the referrer, this is a convinence for adaptors to just leverage PageAction for click analytics without linking it to the respective Page Views.
     */
    refUri?: string;
    /**
     * Time taken in milliseconds since the user saw the page and took the action (such as click on a link, etc.). This will be in seconds.
     */
    timeToAction?: number;
    /**
     * A relative or absolute URL that identifies the page or other item. Defaults to the window location.
     */
    uri?: string;
    /**
     * Page type
     */
    pageType?: string;
    /**
     * Title of the page
     */
    pageName?: string;
    /**
     * Content Id (Parent Id) of the parent in which the content was located;
     */
    parentId?: string;
}

/**
 * PageTags format
 */
declare interface IPageTags {
    /**
     * Meta data tags
     */
    metaTags?: {
        [name: string]: string;
    };
    /**
     * Any other page tag
     */
    [name: string]: string | number | boolean | string[] | number[] | boolean[] | object | undefined;
}

/**
 * Value Callbacks configuration
 */
declare interface IValueCallback {
    /**
     * Function to override the default pageName capturing behavior.
     */
    pageName?: () => string;
    /**
     * A callback function to augument the default pageTags collected during pageAction event.
     */
    pageActionPageTags?: (element?: Element) => IPageTags;
    /**
     * A callback function to populate customized contentName.
     */
    contentName?: (element?: any, useDefaultContentName?: boolean) => string;
}

export { }
